<?php

namespace NoccyLabs\React\Http\Middleware;

use PHPUnit\Framework\Attributes\CoversClass;
use Psr\Http\Message\ResponseInterface;
use Psr\Http\Message\ServerRequestInterface;
use Psr\Log\LoggerInterface;
use Psr\Log\LoggerTrait;
use React\Http\Message\Response;
use React\Http\Message\ServerRequest;
use Stringable;

#[CoversClass(LoggingMiddleware::class)]
class LoggingMiddlewareTest extends \PHPUnit\Framework\TestCase
{
    
    public function testRequestsAreLogged()
    {
        $logger = new TestLogger();
        $middleware = new LoggingMiddleware($logger);

        $request = new ServerRequest("GET", "/", serverParams:['REMOTE_ADDR'=>"127.0.0.1"]);

        $handler = function (ServerRequestInterface $request) {
            return new Response();
        };

        $response = $middleware($request, $handler);

        $this->assertEquals(1, count($logger->logged));
        $this->assertTrue($logger->contains("127.0.0.1"));
    }

    public function testRequestsAreLoggedWithRealIp()
    {
        $logger = new TestLogger();
        $middleware = new LoggingMiddleware($logger);

        $request = new ServerRequest("GET", "/", headers:['X-FORWARDED-FOR'=>"10.1.2.3"], serverParams:['REMOTE_ADDR'=>"127.0.0.1"]);

        $handler = function (ServerRequestInterface $request) {
            return new Response();
        };

        $response = $middleware($request, $handler);

        $this->assertEquals(1, count($logger->logged));
        $this->assertTrue($logger->contains("127.0.0.1 10.1.2.3"));
    }

    public function testRequestsAreLoggedWithUsernames()
    {
        $logger = new TestLogger();
        $middleware = new LoggingMiddleware($logger);

        $request = new ServerRequest("GET", "/", serverParams:['REMOTE_ADDR'=>"127.0.0.1"]);

        $handler = function (ServerRequestInterface $request) {
            return (new Response())->withHeader('x-username', "RandomUser");
        };

        $response = $middleware($request, $handler);

        $this->assertEquals(1, count($logger->logged));
        $this->assertTrue($logger->contains("RandomUser"));
    }

}

class TestLogger implements LoggerInterface
{
    public array $logged = [];

    use LoggerTrait;

    public function log($level, string|Stringable $message, array $context = []): void
    {
        $this->logged[] = $level.": ".$message." ".json_encode($context, JSON_UNESCAPED_SLASHES|JSON_UNESCAPED_UNICODE);
    }

    public function contains(string $string): bool
    {
        foreach ($this->logged as $line) {
            if (str_contains($line, $string)) return true;
        }
        return false;
    }
}
